@@ -4,11 +4,12 @@ from typing import Literal, cast
 from uuid import uuid4
 
 import orjson
+import structlog
 from starlette.exceptions import HTTPException
 from starlette.responses import Response, StreamingResponse
 
 from langgraph_api import config
-from langgraph_api.asyncio import ValueEvent, aclosing
+from langgraph_api.asyncio import ValueEvent
 from langgraph_api.models.run import create_valid_run
 from langgraph_api.route import ApiRequest, ApiResponse, ApiRoute
 from langgraph_api.schema import CRON_FIELDS, RUN_FIELDS
@@ -34,6 +35,8 @@ from langgraph_runtime.database import connect
 from langgraph_runtime.ops import Crons, Runs, Threads
 from langgraph_runtime.retry import retry_db
 
+logger = structlog.stdlib.get_logger(__name__)
+
 
 @retry_db
 async def create_run(request: ApiRequest):
@@ -101,9 +104,7 @@ async def stream_run(
     payload = await request.json(RunCreateStateful)
     on_disconnect = payload.get("on_disconnect", "continue")
     run_id = uuid7()
-    sub = asyncio.create_task(Runs.Stream.subscribe(run_id, thread_id))
-
-    try:
+    async with await Runs.Stream.subscribe(run_id, thread_id) as sub:
         async with connect() as conn:
             run = await create_valid_run(
                 conn,
@@ -113,25 +114,20 @@ async def stream_run(
                 run_id=run_id,
                 request_start_time=request.scope.get("request_start_time_ms"),
             )
-    except Exception:
-        if not sub.cancelled():
-            handle = await sub
-            await handle.__aexit__(None, None, None)
-        raise
 
-    return EventSourceResponse(
-        Runs.Stream.join(
-            run["run_id"],
-            thread_id=thread_id,
-            cancel_on_disconnect=on_disconnect == "cancel",
-            stream_channel=await sub,
-            last_event_id=None,
-        ),
-        headers={
-            "Location": f"/threads/{thread_id}/runs/{run['run_id']}/stream",
-            "Content-Location": f"/threads/{thread_id}/runs/{run['run_id']}",
-        },
-    )
+        return EventSourceResponse(
+            Runs.Stream.join(
+                run["run_id"],
+                thread_id=thread_id,
+                cancel_on_disconnect=on_disconnect == "cancel",
+                stream_channel=sub,
+                last_event_id=None,
+            ),
+            headers={
+                "Location": f"/threads/{thread_id}/runs/{run['run_id']}/stream",
+                "Content-Location": f"/threads/{thread_id}/runs/{run['run_id']}",
+            },
+        )
 
 
 async def stream_run_stateless(
@@ -143,8 +139,7 @@ async def stream_run_stateless(
     on_disconnect = payload.get("on_disconnect", "continue")
     run_id = uuid7()
     thread_id = uuid4()
-    sub = asyncio.create_task(Runs.Stream.subscribe(run_id, thread_id))
-    try:
+    async with await Runs.Stream.subscribe(run_id, thread_id) as sub:
         async with connect() as conn:
             run = await create_valid_run(
                 conn,
@@ -155,26 +150,21 @@ async def stream_run_stateless(
                 request_start_time=request.scope.get("request_start_time_ms"),
                 temporary=True,
             )
-    except Exception:
-        if not sub.cancelled():
-            handle = await sub
-            await handle.__aexit__(None, None, None)
-        raise
 
-    return EventSourceResponse(
-        Runs.Stream.join(
-            run["run_id"],
-            thread_id=run["thread_id"],
-            ignore_404=True,
-            cancel_on_disconnect=on_disconnect == "cancel",
-            stream_channel=await sub,
-            last_event_id=None,
-        ),
-        headers={
-            "Location": f"/runs/{run['run_id']}/stream",
-            "Content-Location": f"/runs/{run['run_id']}",
-        },
-    )
+        return EventSourceResponse(
+            Runs.Stream.join(
+                run["run_id"],
+                thread_id=run["thread_id"],
+                ignore_404=True,
+                cancel_on_disconnect=on_disconnect == "cancel",
+                stream_channel=sub,
+                last_event_id=None,
+            ),
+            headers={
+                "Location": f"/runs/{run['run_id']}/stream",
+                "Content-Location": f"/runs/{run['run_id']}",
+            },
+        )
 
 
 @retry_db
@@ -184,9 +174,7 @@ async def wait_run(request: ApiRequest):
     payload = await request.json(RunCreateStateful)
     on_disconnect = payload.get("on_disconnect", "continue")
     run_id = uuid7()
-    sub = asyncio.create_task(Runs.Stream.subscribe(run_id, thread_id))
-
-    try:
+    async with await Runs.Stream.subscribe(run_id, thread_id) as sub:
         async with connect() as conn:
             run = await create_valid_run(
                 conn,
@@ -196,25 +184,17 @@ async def wait_run(request: ApiRequest):
                 run_id=run_id,
                 request_start_time=request.scope.get("request_start_time_ms"),
             )
-    except Exception:
-        if not sub.cancelled():
-            handle = await sub
-            await handle.__aexit__(None, None, None)
-        raise
 
-    last_chunk = ValueEvent()
+        last_chunk = ValueEvent()
 
-    async def consume():
-        vchunk: bytes | None = None
-        async with aclosing(
-            Runs.Stream.join(
+        async def consume():
+            vchunk: bytes | None = None
+            async for mode, chunk, _ in Runs.Stream.join(
                 run["run_id"],
                 thread_id=run["thread_id"],
-                stream_channel=await sub,
+                stream_channel=sub,
                 cancel_on_disconnect=on_disconnect == "cancel",
-            )
-        ) as stream:
-            async for mode, chunk, _ in stream:
+            ):
                 if (
                     mode == b"values"
                     or mode == b"updates"
@@ -223,43 +203,47 @@ async def wait_run(request: ApiRequest):
                     vchunk = chunk
                 elif mode == b"error":
                     vchunk = orjson.dumps({"__error__": orjson.Fragment(chunk)})
-        if vchunk is not None:
-            last_chunk.set(vchunk)
-        else:
-            async with connect() as conn:
-                thread_iter = await Threads.get(conn, thread_id)
+            if vchunk is not None:
+                last_chunk.set(vchunk)
+            else:
+                async with connect() as conn:
+                    thread_iter = await Threads.get(conn, thread_id)
+                    try:
+                        thread = await anext(thread_iter)
+                        last_chunk.set(thread["values"])
+                    except StopAsyncIteration:
+                        await logger.awarning(
+                            f"No checkpoint found for thread {thread_id}",
+                            thread_id=thread_id,
+                        )
+                        last_chunk.set(b"{}")
+
+        # keep the connection open by sending whitespace every 5 seconds
+        # leading whitespace will be ignored by json parsers
+        async def body() -> AsyncIterator[bytes]:
+            stream = asyncio.create_task(consume())
+            while True:
                 try:
-                    thread = await anext(thread_iter)
-                    last_chunk.set(thread["values"])
-                except StopAsyncIteration:
-                    last_chunk.set(b"{}")
+                    if stream.done():
+                        # raise stream exception if any
+                        stream.result()
+                    yield await asyncio.wait_for(last_chunk.wait(), timeout=5)
+                    break
+                except TimeoutError:
+                    yield b"\n"
+                except asyncio.CancelledError:
+                    stream.cancel()
+                    await stream
+                    raise
 
-    # keep the connection open by sending whitespace every 5 seconds
-    # leading whitespace will be ignored by json parsers
-    async def body() -> AsyncIterator[bytes]:
-        stream = asyncio.create_task(consume())
-        while True:
-            try:
-                if stream.done():
-                    # raise stream exception if any
-                    stream.result()
-                yield await asyncio.wait_for(last_chunk.wait(), timeout=5)
-                break
-            except TimeoutError:
-                yield b"\n"
-            except asyncio.CancelledError:
-                stream.cancel()
-                await stream
-                raise
-
-    return StreamingResponse(
-        body(),
-        media_type="application/json",
-        headers={
-            "Location": f"/threads/{thread_id}/runs/{run['run_id']}/join",
-            "Content-Location": f"/threads/{thread_id}/runs/{run['run_id']}",
-        },
-    )
+        return StreamingResponse(
+            body(),
+            media_type="application/json",
+            headers={
+                "Location": f"/threads/{thread_id}/runs/{run['run_id']}/join",
+                "Content-Location": f"/threads/{thread_id}/runs/{run['run_id']}",
+            },
+        )
 
 
 @retry_db
@@ -270,9 +254,7 @@ async def wait_run_stateless(request: ApiRequest):
     on_disconnect = payload.get("on_disconnect", "continue")
     run_id = uuid7()
     thread_id = uuid4()
-    sub = asyncio.create_task(Runs.Stream.subscribe(run_id, thread_id))
-
-    try:
+    async with await Runs.Stream.subscribe(run_id, thread_id) as sub:
         async with connect() as conn:
             run = await create_valid_run(
                 conn,
@@ -283,25 +265,18 @@ async def wait_run_stateless(request: ApiRequest):
                 request_start_time=request.scope.get("request_start_time_ms"),
                 temporary=True,
             )
-    except Exception:
-        if not sub.cancelled():
-            handle = await sub
-            await handle.__aexit__(None, None, None)
-        raise
-    last_chunk = ValueEvent()
 
-    async def consume():
-        vchunk: bytes | None = None
-        async with aclosing(
-            Runs.Stream.join(
+        last_chunk = ValueEvent()
+
+        async def consume():
+            vchunk: bytes | None = None
+            async for mode, chunk, _ in Runs.Stream.join(
                 run["run_id"],
                 thread_id=run["thread_id"],
-                stream_channel=await sub,
+                stream_channel=sub,
                 ignore_404=True,
                 cancel_on_disconnect=on_disconnect == "cancel",
-            )
-        ) as stream:
-            async for mode, chunk, _ in stream:
+            ):
                 if (
                     mode == b"values"
                     or mode == b"updates"
@@ -310,38 +285,43 @@ async def wait_run_stateless(request: ApiRequest):
                     vchunk = chunk
                 elif mode == b"error":
                     vchunk = orjson.dumps({"__error__": orjson.Fragment(chunk)})
-        if vchunk is not None:
-            last_chunk.set(vchunk)
-        else:
-            # we can't fetch the thread (it was deleted), so just return empty values
-            last_chunk.set(b"{}")
+            if vchunk is not None:
+                last_chunk.set(vchunk)
+            else:
+                # we can't fetch the thread (it was deleted), so just return empty values
+                await logger.awarning(
+                    "No checkpoint emitted for stateless run",
+                    run_id=run["run_id"],
+                    thread_id=run["thread_id"],
+                )
+                last_chunk.set(b"{}")
 
-    # keep the connection open by sending whitespace every 5 seconds
-    # leading whitespace will be ignored by json parsers
-    async def body() -> AsyncIterator[bytes]:
-        stream = asyncio.create_task(consume())
-        while True:
-            try:
-                if stream.done():
-                    # raise stream exception if any
-                    stream.result()
-                yield await asyncio.wait_for(last_chunk.wait(), timeout=5)
-                break
-            except TimeoutError:
-                yield b"\n"
-            except asyncio.CancelledError:
-                stream.cancel("Run stream cancelled")
-                await stream
-                raise
+        # keep the connection open by sending whitespace every 5 seconds
+        # leading whitespace will be ignored by json parsers
+        async def body() -> AsyncIterator[bytes]:
+            stream = asyncio.create_task(consume())
+            while True:
+                try:
+                    if stream.done():
+                        # raise stream exception if any
+                        stream.result()
+                    yield await asyncio.wait_for(last_chunk.wait(), timeout=5)
+                    break
+                except TimeoutError:
+                    yield b"\n"
+                except asyncio.CancelledError:
+                    stream.cancel("Run stream cancelled")
+                    await stream
+                    raise
 
-    return StreamingResponse(
-        body(),
-        media_type="application/json",
-        headers={
-            "Location": f"/threads/{run['thread_id']}/runs/{run['run_id']}/join",
-            "Content-Location": f"/threads/{run['thread_id']}/runs/{run['run_id']}",
-        },
-    )
+        return StreamingResponse(
+            body(),
+            media_type="application/json",
+            headers={
+                "Location": f"/threads/{run['thread_id']}/runs/{run['run_id']}/join",
+                "Content-Location": f"/threads/{run['thread_id']}/runs/{run['run_id']}",
+            },
+        )
 
 
 @retry_db
