@@ -153,7 +153,7 @@ async def worker(
                 raise UserTimeout(e) from e
             raise
 
-    async with Runs.enter(run_id, run["thread_id"], main_loop) as done:
+    async with Runs.enter(run_id, run["thread_id"], main_loop, resumable) as done:
         # attempt the run
         try:
             if attempt > BG_JOB_MAX_RETRIES:
